/* <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    MotionPoint Class

    動くポイント

>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> */


package com.bigchu.display {

    import flash.display.MovieClip;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import caurina.transitions.Tweener;

    public class MotionPoint extends MovieClip {

        public static var TYPE_BEZIER_POINT: String = "typeBezierPoint";
        public static var TYPE_CTRL_POINT: String = "typeCtrlPoint";

		private var _px: Number = 0;
		private var _py: Number = 0;
		private var _pz: Number = 0;
        
		private var _speedX: Number = 0;
		private var _speedY: Number = 0;
		private var _speedZ: Number = 0;

        private var _offsetX: Number = 0;
        private var _offsetY: Number = 0;
        private var _offsetZ: Number = 0;

        private var _dirX: int;
        private var _dirY: int;
        private var _dirZ: int;

        private var _enabled: Boolean;

        public function set px (n: Number): void { _px = n; x = n;}
        public function set py (n: Number): void { _py = n; y = n;}
        public function set pz (n: Number): void { _pz = n; scaleX = scaleY = n;}
        public function set speedX (n: Number): void { _speedX = n; }
        public function set speedY (n: Number): void { _speedY = n; }
        public function set speedZ (n: Number): void { _speedZ = n; }
        public function set offsetX (n: Number): void { _offsetX = n; }
        public function set offsetY (n: Number): void { _offsetY = n; }
        public function set offsetZ (n: Number): void { _offsetZ = n; }
        public function set dirX (n: int): void { _dirX = n; }
        public function set dirY (n: int): void { _dirY = n; }
        public function set dirZ (n: int): void { _dirZ = n; }

        public function get px (): Number { return(_px); }
        public function get py (): Number { return(_py); }
        public function get pz (): Number { return(_pz); }
        public function get speedX (): Number { return(_speedX); }
        public function get speedY (): Number { return(_speedY); }
        public function get speedZ (): Number { return(_speedZ); }
        public function get offsetX (): Number { return(_offsetX); }
        public function get offsetY (): Number { return(_offsetY); }
        public function get offsetZ (): Number { return(_offsetZ); }
        public function get dirX (): int { return(_dirX); }
        public function get dirY (): int { return(_dirY); }
        public function get dirZ (): int { return(_dirZ); }

        override public function set enabled (b: Boolean): void {
            if (b) addButtonHandlr();
        }
        override public function get enabled (): Boolean { return(_enabled); }



        /**
         * Constructor
         * @param inX: X
         * @param inY: Y
         * @param inZ: Z 
        **/
		public function MotionPoint (
            
            inType: String,
            inX: Number = 0,
            inY: Number = 0,
            inZ: Number = 1,
            inR: Number = 0,
            inGom: Boolean = false,
            inEnabled: Boolean = true
        
        ) {
            
            _px = inX;
            _py = inY;
            _pz = inZ;

            _offsetX = inX;
            _offsetY = inY;
            _offsetZ = inZ;
            
            init();

            if (inGom)
                addButtonHandlrGom();
            else
                enabled = inEnabled;
            
            graphics.lineStyle(0, 0, 0);

            switch (inType) {

                case TYPE_BEZIER_POINT:
                    graphics.beginFill(0Xc09f44);
                    graphics.drawCircle(0, 0, inR);
                    break;

                case TYPE_CTRL_POINT:
                    graphics.beginFill(0xab4109);
                    graphics.drawCircle(0, 0, inR);
                    break;

                default:
                    graphics.beginFill(0x000000);
                    graphics.drawCircle(0, 0, 0);
                    break;
            }
			graphics.endFill();

        }






        /**
         * init
         */
        private function init (): void {
            
            x = _px;
            y = _py;
            scaleX = scaleY = _pz;
            
            addEventListener(Event.ENTER_FRAME, function (): void {
                _px += _dirX * _speedX;
                _py += _dirY * _speedY;
                _pz += _dirZ * _speedZ;
            });
            addEventListener(Event.ENTER_FRAME, setPosition);
			
        }






        /**
         * buttonEvents
         */
        private function addButtonHandlr (): void {
            addEventListener(MouseEvent.MOUSE_DOWN, function (): void {
                startDrag();
                removeEventListener(Event.ENTER_FRAME, setPosition);
            });
            addEventListener(MouseEvent.MOUSE_UP, function (): void {
                stopDrag();
                _px = x;
                _py = y;
                _pz = scaleX;
                addEventListener(Event.ENTER_FRAME, setPosition);
            });
            buttonMode = true;
        }






        /**
         * buttonEvents
         */
        private function addButtonHandlrGom (): void {
            addEventListener(MouseEvent.MOUSE_DOWN, function (): void {
                startDrag();
                removeEventListener(Event.ENTER_FRAME, setPosition);
            });
            addEventListener(MouseEvent.MOUSE_UP, function (): void {
                stopDrag();
                gomAnime();
            });
            buttonMode = true;
        }






        /**
         * setPosition
         */
        private function setPosition (e: Event): void {
            x = _px;
            y = _py;
            scaleX = scaleY = _pz;
        }






        /**
         * gomAnime
         */
        private function gomAnime (): void {
            
            Tweener.addTween(this, {
                x: _offsetX,
                y: _offsetY,
                scaleX: _offsetZ,
                scaleY: _offsetZ,
                time: 1.2,
                delay: 0,
                transition: "easeOutElastic",
                onComplete: function (): void {
                    _px = x;
                    _py = y;
                    _pz = scaleX;
                    addEventListener(Event.ENTER_FRAME, setPosition);
                }
            });

        }


        /**
         * gomAnime
         */
        public function roopAnime (
            minX: Number,
            minY: Number,
            maxX: Number,
            maxY: Number,
            inTime: Number,
            inTType: String
        ): void {
            
            removeEventListener(Event.ENTER_FRAME, setPosition);
            
            Tweener.addTween(this, {
                x: minX,
                y: minY,
                time: inTime,
                transition: inTType,
                onComplete: function (): void {
                    Tweener.addTween(this, {
                        x: maxX,
                        y: maxY,
                        time: inTime,
                        transition: inTType,
                        onComplete: function (): void {
                            roopAnime(minX, minY, maxX, maxY, inTime, inTType);
                        }
                    });
                }
            });

        }


	}
}
